/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail;

import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.TagSupport;

/**
 * <p>
 * This is an abstract implementation of the NodeDiscovery interface and is useful to NodeDiscovery
 * implementations that are also a Tag.
 * </p>
 *
 * <p>
 * This abstract class simplifies the work required when an implementation of the NodeDiscovery will also
 * provide services as a tag and provides an attribute to set the title.
 * </p>
 *
 * <p>
 * <b>Note: </b> It is strongly recommend that implementations of NodeDiscovery, who also serve as Tags,
 * inherit from this class. The subclass then does not need to worry about tag rendering.
 * </p>
 *
 * <p>
 * <b>Thread Safety:</b> This class is thread safe in the context of the JSP rendering process.
 * </p>
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 *
 */
public abstract class AbstractNodeDiscovery extends TagSupport implements NodeDiscovery {
    /**
     * <p>
     * Represents the title associated with this node.
     * </p>
     *
     * <p>
     * This variable is set initially to null, is mutable and can be null but never an empty string.
     * A null title means that the title for the current node should default that found in the
     * PathDiscovery data source.
     * </p>
     *
     * <p>
     * This variable is referenced by the associated getter/setter methods
     * </p>
     */
    private String title;

    /**
     * <p>
     * Constructs this abstract base for NodeDiscovery.
     * </p>
     *
     */
    protected AbstractNodeDiscovery() {
        // empty
    }

    /**
     * <p>
     * This method will get the current node url and if found, save it to BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE
     * attribute using page scope.
     * </p>
     *
     * @return SKIP_BODY to skip body evaluation
     *
     * @throws BreadCrumbException if failed to discovery the current node
     *
     */
    public int doStartTag() throws BreadCrumbException {
        BreadCrumbNode node = getCurrentNode(pageContext);
        if (node != null) {
            pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE, node, PageContext.PAGE_SCOPE);
        }

        return SKIP_BODY;
    }

    /**
     * <p>
     * Returns value of title property.
     * </p>
     *
     * @return The possibly null, non-empty string representing the title
     *
     */
    public String getTitle() {
        return title;
    }

    /**
     * <p>
     * Sets value of title property.
     * </p>
     *
     * <p>
     * <b>Note: </b> If the given title is an empty string, null will be used instead.
     * </p>
     *
     * @param title A possibly null, possibly empty (trim'd) string representing the title
     *
     */
    public void setTitle(String title) {
        this.title = (Util.isEmptyString(title) ? null : title);
    }
}
