/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail;

import javax.servlet.jsp.tagext.TagData;
import javax.servlet.jsp.tagext.TagExtraInfo;
import javax.servlet.jsp.tagext.VariableInfo;

/**
 * <p>
 * This is the tag extra info that describes the attribute to variable mapping that will occur within the
 * BreadCrumbTrailTag.
 * </p>
 *
 * <p>
 * This tag extra info will define two variables (that can be used in the body of the BreadCrumbTrailTag):
 * <ol>
 * <li>BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE variable that will contain a reference to the BreadCrumbNode that
 * should be formatted</li>
 * <li>BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS variable that will contain a reference to the String that
 * represents the style that will be used.  Please note that this string can be an empty or null string to represent
 * no styles to be applied</li>
 * </ol>
 * </p>
 *
 * <p>
 * <b>Note: </b> This is the tag extra info class that should be specified in the corresponding tld file.
 * </p>
 *
 * <p>
 * <b>Thread Safety: </b> This class is immutable and thread safe.
 * </p>
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 *
 * @since 2.0
 *
 */
public class BreadCrumbTrailTagExtraInfo extends TagExtraInfo {
    /**
     * <p>
     * This method will define the variables that can be used in the body of the BreadCrumbTrailTag.
     * </p>
     *
     * <p>
     * <b>Notes: </b> The TagData argument is never used and not validations will be done on it.
     * </p>
     *
     * @param data A non-null tag data to use (not used)
     *
     * @return Always a 2 element array representing the two elements to define
     */
    public VariableInfo[] getVariableInfo(TagData data) {
        return new VariableInfo[] {
            new VariableInfo(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, BreadCrumbNode.class.getName(), true,
                VariableInfo.NESTED),
            new VariableInfo(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, String.class.getName(), true,
                VariableInfo.NESTED)
        };
    }
}
