/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail;

import javax.servlet.jsp.PageContext;


/**
 * <p>
 * This interface defines the contract for classes wishing to discover the current node.
 * </p>
 *
 * <p>
 * Implementations of this interface will be called when the current page has not been defined (i.e. a page scope
 * attribute for BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE).
 * The implementation should discover, in it's own way, the current node when getCurrentNode() is called and return
 * the node representation of the current page.
 * </p>
 *
 * <p>
 * <b>Note: </b> Implementations of this interface will be created in the constructor of the BreadCrumbTrailTag via
 * the object factory when the page scope attribute (BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE)
 * has not already been defined. The node representation of the current page will then be returned by the implementing
 * class and used to discover the path to the root.
 * </p>
 *
 * <p>
 * <b>Thread Safety:</b> Implementations should be thread safe in the context of the JSP rendering process.
 * </p>
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 *
 */
public interface NodeDiscovery {
    /**
     * <p>
     * Called to retrieve the node representation of the current page. If the current page could not be found or
     * is not applicable - simply return null. The returned node should include any query strings available.
     * </p>
     *
     * @param pageContext A non-null page context to use
     * @return A non-null Node representing the current url or null if the node couldn't be determined
     *
     * @throws IllegalArgumentException if the page context is null
     *
     * @throws BreadCrumbException if an exception occurs discovering the current page
     */
    public BreadCrumbNode getCurrentNode(PageContext pageContext) throws BreadCrumbException;
}
