/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;

import java.util.Map;


/**
 * <p>
 * Defines the contract for a data source for the site.
 * </p>
 *
 * <p>
 * Implementations of this interface should create and return a site map consisting of NodeMatcher nodes
 * that describe the site map. The returns map should have each unique NodeMatcher listed as the key and an
 * List of NodeMatcher nodes that describe the children of that key.
 * </p>
 *
 * <p>
 * <b>Thread Safety:</b> Implementations of this interface should be thread safe.
 * </p>
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 *
 */
public interface SiteDiscovery {
    /**
     * <p>
     * Returns a map of NodeMatcher keys with a List of NodeMatcher elements as the value.
     * The key NodeMatcher represents the parent and the value (the list) represents the child values.
     * The value can be null or an empty list to represent no child elements (i.e. it's a leaf node).
     * </p>
     *
     * @return A non-null map of parent NodeMatcher to a list of NodeMatcher children
     *
     * @throws BreadCrumbException if an exception occurs retrieving the site map
     */
    public Map getSiteMap() throws BreadCrumbException;
}
