/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail;

import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.Tag;

import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.framework.Test;

/**
 * <p>
 * Unit test cases for AbstractNodeDiscovery.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0
 */
public class AbstractNodeDiscoveryTests extends TestCase {
    /**
     * <p>
     * BreadCrumbNode instance to act as the current node discoveryed in SimpleNodeDiscovery instance.
     * </p>
     *
     */
    private BreadCrumbNode node;

    /**
     * <p>
     * PageContext instance for calling getCurrentNode(PageContext) method.
     * </p>
     *
     */
    private PageContext pageContext;

    /**
     * <p>
     * AbstractNodeDiscovery instance for testing.
     * </p>
     *
     */
    private AbstractNodeDiscovery nodeDiscovery;

    /**
     * <p>
     * Setup test environment.
     * </p>
     *
     */
    protected void setUp() {
        node = new BreadCrumbNode("/main", "Main");
        pageContext = new MockPageContext();
        nodeDiscovery = new SimpleNodeDiscovery(node);
        nodeDiscovery.setPageContext(pageContext);
    }

    /**
     * <p>
     * Tears down test environment.
     * </p>
     *
     */
    protected void tearDown() {
        pageContext = null;
        nodeDiscovery = null;
    }

    /**
     * <p>
     * Return all tests.
     * </p>
     *
     * @return all tests
     */
    public static Test suite() {
        return new TestSuite(AbstractNodeDiscoveryTests.class);
    }

    /**
     * <p>
     * Tests AbstractNodeDiscovery#doStartTag() for accuracy.
     * Expects the BREADCRUMB_CURRENT_NODE is correctly set after calling doStartTag().
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testDoStartTag() throws Exception {
        int value = nodeDiscovery.doStartTag();

        assertEquals("Expects the int value that SKIP_BODY represents.", Tag.SKIP_BODY, value);

        BreadCrumbNode currentNode = (BreadCrumbNode) pageContext.getAttribute(
                BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE, PageContext.PAGE_SCOPE);

        assertEquals("Expects equal title.", node.getTitle(), currentNode.getTitle());
        assertEquals("Expects equal url.", node.getUrl(), currentNode.getUrl());
    }

    /**
     * <p>
     * Tests AbstractNodeDiscovery#doStartTag() for accuracy.
     * It test the case when getCurrentNode() returns null.
     * Expects the BREADCRUMB_CURRENT_NODE attribute is null after calling doStartTag().
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testDoStartTag2() throws Exception {
        nodeDiscovery = new SimpleNodeDiscovery(null);
        nodeDiscovery.setPageContext(pageContext);

        int value = nodeDiscovery.doStartTag();

        assertEquals("Expects the int value that SKIP_BODY represents.", Tag.SKIP_BODY, value);

        assertNull("Expects null current node in page context.", pageContext.getAttribute(
                BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE, PageContext.PAGE_SCOPE));
    }

    /**
     * <p>
     * Tests AbstractNodeDiscovery#getTitle() for accuracy.
     * Expects title is initially null.
     * </p>
     */
    public void testGetTitle() {
        assertNull("Expect title is initially null.", nodeDiscovery.getTitle());
    }

    /**
     * <p>
     * Tests AbstractNodeDiscovery#setTitle(String) for accuracy.
     * Expects the title is correctly set.
     * </p>
     */
    public void testSetTitle() {
        nodeDiscovery.setTitle("Main");

        assertEquals("Failed to get correct title.", "Main", nodeDiscovery.getTitle());
    }

    /**
     * <p>
     * Tests AbstractNodeDiscovery#setTitle(String) for accuracy.
     * It tests the case that when title is null and expects success.
     * </p>
     */
    public void testSetTitle_NullTitle() {
        nodeDiscovery.setTitle(null);

        assertNull("Expect title is null.", nodeDiscovery.getTitle());
    }

    /**
     * <p>
     * Tests AbstractNodeDiscovery#setTitle(String) for accuracy.
     * It tests the case that when title is empty and expects success.
     * </p>
     */
    public void testSetTitle_EmptyTitle() {
        nodeDiscovery.setTitle(" ");

        assertNull("Expect title is null.", nodeDiscovery.getTitle());
    }

    /**
     * <p>
     * This class extends AbstractNodeDiscovery class.
     * It is used for testing propose.
     * </p>
     *
     * @author TCSDEVELOPER
     * @version 2.0
     */
    private static final class SimpleNodeDiscovery extends AbstractNodeDiscovery {
        /**
         * <p>
         * Saves the current BreadCrumbNode instance.
         * It is set by constructor and retrieved by getCurrentNode(PageContext) method.
         * </p>
         */
        private BreadCrumbNode node;

        /**
         * <p>
         * Constructs a new SimpleNodeDiscovery instance.
         * </p>
         *
         * @param node BreadCrumbNode instance to be retrieved by getCurrentNode(PageContext) method.
         */
        public SimpleNodeDiscovery(BreadCrumbNode node) {
            this.node = node;
        }

        /**
         * <p>
         * Returns the current BreadCrumbNode instance discoveried by this instance.
         * </p>
         *
         * @param pageContext PageContext instance, not used.
         *
         * @return the current BreadCrumbNode instance discoveried by this instance.
         */
        public BreadCrumbNode getCurrentNode(PageContext pageContext) {
            return node;
        }

    }
}