/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.apache.cactus.JspTestCase;
import org.apache.cactus.WebResponse;
import org.apache.cactus.extension.jsp.JspTagLifecycle;

import com.topcoder.web.ui.tag.breadcrumbtrail.nodeformatter.HtmlAnchorNodeFormatterTag;

/**
 * <p>
 * In-container test cases for BreadCrumbTrailTag.
 * </p>
 *
 * <p>
 * For unit test cases, please see BreadCrumbTrailTagTests.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0
 *
 */
public class BreadCrumbTrailTagInContainerTests extends JspTestCase {
    /**
     * <p>
     * BreadCrumbTrailTag instance for testing.
     * </p>
     *
     */
    private BreadCrumbTrailTag tag;

    /**
     * <p>
     * JspTagLifecycle instance for executing tag life cycle.
     * </p>
     *
     */
    private JspTagLifecycle lifecycle;

    /**
     * <p>
     * Setup test environment.
     * </p>
     *
     */
    protected void setUp() {
        tag = new BreadCrumbTrailTag();
        lifecycle = new JspTagLifecycle(pageContext, tag);
    }

    /**
     * <p>
     * Tears down test environment.
     * </p>
     *
     */
    protected void tearDown() {
        tag = null;
        lifecycle = null;
    }

    /**
     * <p>
     * Return all tests.
     * </p>
     *
     * @return all tests
     */
    public static Test suite() {
        return new TestSuite(BreadCrumbTrailTagInContainerTests.class);
    }

    /**
     * <p>
     * In this test, it tests the case when pathOverride is specified and BREADCRUMB_CURRENT_NODE attribute is set.
     * The tag body is static text.
     * Expects the tag body is evaluated there times because the current path contains there elements.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testDoAfterBodyNoStyle() throws Exception {
        BreadCrumbNode node = new BreadCrumbNode("/topics", "Topics");
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE, node);

        tag.setPathOverride(TestHelper.getPathOverride());
        lifecycle.addNestedText("NODE");

        lifecycle.expectBodyEvaluated(3);
        lifecycle.invoke();
    }

    /**
     * <p>
     * Verifies the response text content.
     * </p>
     *
     * @param response the WebResponse instance
     */
    public void endDoAfterBodyNoStyle(WebResponse response) {
        assertEquals("Failed to generate correct bread crumb trail.", "NODE:NODE:NODE", response.getText());
    }

    /**
     * <p>
     * In this test, it tests the case when pathOverride is specified and BREADCRUMB_CURRENT_NODE attribute is set.
     * The tag body is another tag(HtmlAnchorNodeFormatterTag).
     * Expects the tag body is evaluated there times because the current path contains there elements.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testDoAfterBodyWithNestedTag() throws Exception {
        BreadCrumbNode node = new BreadCrumbNode("/topics", "Topics");
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE, node);

        tag.setPathOverride(TestHelper.getPathOverride());
        HtmlAnchorNodeFormatterTag formatterTag = new HtmlAnchorNodeFormatterTag();
        lifecycle.addNestedTag(formatterTag);

        lifecycle.expectBodyEvaluated(3);
        lifecycle.invoke();
    }

    /**
     * <p>
     * Verifies the response text content.
     * </p>
     *
     * @param response the WebResponse instance
     */
    public void endDoAfterBodyWithNestedTag(WebResponse response) {
        String expectedHtml = "<a class=\"root-node-style\" href=\"/main\">Main</a>:<a class=\"node-style\" "
            + "href=\"/forums\">Help</a>:<a class=\"current-node-style\" href=\"/topics\">Topics</a>";
        assertEquals("Failed to generate correct bread crumb trail.", expectedHtml, response.getText());
    }

    /**
     * <p>
     * In this test, it tests the case when pathOverride is specified, BREADCRUMB_CURRENT_NODE attribute is set
     * and all the styles are set.
     * The tag body is another tag(HtmlAnchorNodeFormatterTag).
     * Expects the tag body is evaluated there times because the current path contains there elements.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testDoAfterBodyWithStyle() throws Exception {
        BreadCrumbNode node = new BreadCrumbNode("/topics", "Topics");
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE, node);

        tag.setPathOverride(TestHelper.getPathOverride());

        tag.setNodeStyle("font-size: 20pt");
        tag.setRootNodeStyle("font-size: 10pt;font-weight: bold");
        tag.setCurrentNodeStyle("font-size: 30pt;font-weight: bold");
        tag.setMouseOverStyle("color: yellow");
        tag.setPathSeparatorStyle("color: green");

        HtmlAnchorNodeFormatterTag formatterTag = new HtmlAnchorNodeFormatterTag();
        lifecycle.addNestedTag(formatterTag);

        lifecycle.expectBodyEvaluated(3);
        lifecycle.invoke();
    }

    /**
     * <p>
     * Verifies the response text content.
     * </p>
     *
     * @param response the WebResponse instance
     */
    public void endDoAfterBodyWithStyle(WebResponse response) {
        StringBuffer expectedHtml = new StringBuffer();
        expectedHtml.append("<span><style>\n");
        expectedHtml.append(".path-separator-style {color: green}\n");
        expectedHtml.append("a.node-style:link {font-size: 20pt}\n");
        expectedHtml.append("a.current-node-style:link {font-size: 30pt;font-weight: bold}\n");
        expectedHtml.append("a.root-node-style:link {font-size: 10pt;font-weight: bold}\n");
        expectedHtml.append("a.node-style:hover {color: yellow}\n");
        expectedHtml.append("a.current-node-style:hover {color: yellow}\n");
        expectedHtml.append("a.root-node-style:hover {color: yellow}\n");
        expectedHtml.append("</style>\n");
        expectedHtml.append(
                "<a class=\"root-node-style\" href=\"/main\">Main</a><span class=\"path-separator-style\">:</span>\n");
        expectedHtml.append(
                "<a class=\"node-style\" href=\"/forums\">Help</a><span class=\"path-separator-style\">:</span>\n");
        expectedHtml.append("<a class=\"current-node-style\" href=\"/topics\">Topics</a></span>");

        assertEquals("Failed to generate correct bread crumb trail.", expectedHtml.toString(), response.getText());
    }

    /**
     * <p>
     * In this test, it tests the case when BREADCRUMB_CURRENT_NODE attribute is set, but pathOverride is not specified,
     * so the PathDiscovery instance is created via object factory.
     * The tag body is another tag(HtmlAnchorNodeFormatterTag).
     * Expects the tag body is evaluated two times because the current path contains two elements.
     * </p>
     *
     * <p>
     * The site map graph:
     * &lt;bc_node title=&quot;MyMain&quot; url=&quot;/&quot; pattern=&quot;/&quot;&gt;
     *     &lt;bc_node title=&quot;Forums&quot; url=&quot;/forums&quot; pattern=&quot;/forums&quot;&gt;
     *        &lt;bc_node title=&quot;Help&quot; url=&quot;/forums&quot; pattern=&quot;/forums[?]id.*&quot;/&gt;
     *     &lt;/bc_node&gt;
     *     &lt;bc_node title=&quot;Topics&quot; url=&quot;/topics&quot; pattern=&quot;/topics&quot;&gt;
     *         &lt;bc_node title=&quot;Help&quot; url=&quot;/forums&quot; pattern=&quot;/forums[?]id.*&quot;/&gt;
     *         &lt;bc_node title=&quot;Questions&quot; url=&quot;/questions&quot; pattern=&quot;/questions&quot;/&gt;
     *     &lt;/bc_node&gt;&gt;
     * &lt;/bc_node&gt;
     *
     * &lt;bc_node title=&quot;AltMain&quot; url=&quot;/main&quot; pattern=&quot;/&quot;&gt;
     *     &lt;bc_node title=&quot;Help&quot; url=&quot;/forums&quot; pattern=&quot;/forums[?]id.*&quot;/&gt;
     *     &lt;bc_node title=&quot;Topics&quot; url=&quot;/topics&quot; pattern=&quot;/topics&quot;/&gt;
     * &lt;/bc_node&gt;
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testDoAfterBodyUsingObjectFactory() throws Exception {
        String configPath = request.getRealPath("/config/config.xml");
        TestHelper.loadMultipleXMLConfig(BreadCrumbTrailTag.class.getName(), configPath);

        BreadCrumbNode node = new BreadCrumbNode("/forums", "Forums");
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE, node);

        HtmlAnchorNodeFormatterTag formatterTag = new HtmlAnchorNodeFormatterTag();
        lifecycle.addNestedTag(formatterTag);

        lifecycle.expectBodyEvaluated(2);
        lifecycle.invoke();

        TestHelper.clearConfigFile(BreadCrumbTrailTag.class.getName());
    }

    /**
     * <p>
     * Verifies the response text content.
     * </p>
     *
     * @param response the WebResponse instance
     */
    public void endDoAfterBodyUsingObjectFactory(WebResponse response) {
        String expectedHtml = "<a class=\"root-node-style\" href=\"/\">MyMain</a>:<a class=\"current-node-style\" "
            + "href=\"/forums\">Forums</a>";
        assertEquals("Failed to generate correct bread crumb trail.", expectedHtml, response.getText());
    }
}
