/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.servlet.jsp.PageContext;

import com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery.nodematchers.RegexNodeMatcher;

import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.framework.Test;

/**
 * <p>
 * Unit test cases for Util.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0
 */
public class UtilTests extends TestCase {

    /**
     * <p>
     * Return all tests.
     * </p>
     *
     * @return all tests
     */
    public static Test suite() {
        return new TestSuite(UtilTests.class);
    }

    /**
     * <p>
     * Tests Util#checkNull(Object,String) method.
     * It test the case when a not null object is passed in and expects success.
     * </p>
     */
    public void testCheckNull_NotNullObject() {
        Util.checkNull("", "test");
    }

    /**
     * <p>
     * Tests Util#checkNull(Object,String) method.
     * It test the case when a null object is passed in and expects IllegalArgumentException.
     * </p>
     */
    public void testCheckNull_NullObject() {
        try {
            Util.checkNull(null, "test");
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests Util#checkString(String,String) method.
     * It test the case when a not null and not empty string is passed in and expects success.
     * </p>
     */
    public void testCheckString_NotNullNotEmptyString() {
        Util.checkString("test", "test");
    }

    /**
     * <p>
     * Tests Util#checkString(String,String) method.
     * It test the case when a null string is passed in and expects IllegalArgumentException.
     * </p>
     */
    public void testCheckString_NullString() {
        try {
            Util.checkString(null, "test");
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests Util#checkString(String,String) method.
     * It test the case when an empty string is passed in and expects IllegalArgumentException.
     * </p>
     */
    public void testCheckString_EmptyString() {
        try {
            Util.checkString("", "test");
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests Util#isEmptyString(String) for accuracy.
     * </p>
     */
    public void testIsEmptyString_NullString() {
        assertFalse("Expects false when the argument is null string.", Util.isEmptyString(null));
    }

    /**
     * <p>
     * Tests Util#isEmptyString(String) for accuracy.
     * </p>
     */
    public void testIsEmptyString_EmptyString() {
        assertTrue("Expects true when the argument is empty string.", Util.isEmptyString("  "));
    }

    /**
     * <p>
     * Tests Util#isEmptyString(String) for accuracy.
     * </p>
     */
    public void testIsEmptyString_NotNullNotEmptyString() {
        assertFalse("Expects false when the argument is non-null and non-empty string.", Util.isEmptyString("test"));
    }

    /**
     * <p>
     * Tests Util#shortestPath(Map,BreadCrumbNode) for accuracy.
     * </p>
     *
     * <p>
     * It tests the case when the current node doesn't match any node in site map.
     * Expects null be returned.
     * </p>
     *
     * <p>
     * The siteMap graph :
     * &lt;bc_node title=&quot;MyMain&quot; url=&quot;/&quot; pattern=&quot;/&quot;&gt;
     *     &lt;bc_node title=&quot;Topics&quot; url=&quot;/topics&quot; pattern=&quot;/topics&quot;/&gt;
     * &lt;/bc_node&gt;
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testShortestPath1() throws Exception {
        Map siteMap = TestHelper.getSiteMap("test_files/path/path1.xml");
        BreadCrumbNode node = new BreadCrumbNode("/Forums", "Forums");

        List pathList = Util.shortestPath(siteMap, node);

        assertNull("Expects no shortest path found.", pathList);
    }

    /**
     * <p>
     * Tests Util#shortestPath(Map,BreadCrumbNode) for accuracy.
     * </p>
     *
     * <p>
     * It tests the case when the site map doesn't contain root nodes.
     * Expects null be returned.
     * </p>
     *
     * <p>
     * The siteMap graph :
     * &lt;bc_node title=&quot;MyMain&quot; url=&quot;/&quot; pattern=&quot;/&quot;&gt;
     *     &lt;bc_node title=&quot;Topics&quot; url=&quot;/topics&quot; pattern=&quot;/topics&quot;&gt;
     *         &lt;bc_node title=&quot;MyMain&quot; url=&quot;/&quot; pattern=&quot;/&quot; /&gt;
     *     &lt;/bc_node&gt;
     * &lt;/bc_node&gt;
     * &lt;/p&gt;
     *
     * @throws Exception to JUnit.
     */
    public void testShortestPath2() throws Exception {
        Map siteMap = TestHelper.getSiteMap("test_files/path/path2.xml");
        BreadCrumbNode node = new BreadCrumbNode("/", "MyMain");

        List pathList = Util.shortestPath(siteMap, node);

        assertNull("Expects no shortest path found.", pathList);
    }

    /**
     * <p>
     * Tests Util#shortestPath(Map,BreadCrumbNode) for accuracy.
     * </p>
     *
     * <p>
     * Expects one path be returned while there's only one path is available.
     * </p>
     *
     * <p>
     * The siteMap graph :
     * &lt;bc_node title=&quot;MyMain&quot; url=&quot;/&quot; pattern=&quot;/&quot;&gt;
     *     &lt;bc_node title=&quot;Topics&quot; url=&quot;/topics&quot; pattern=&quot;/topics&quot;&gt;
     *         &lt;bc_node title=&quot;Forums&quot; url=&quot;/Forums&quot; pattern=&quot;/Forums&quot; /&gt;
     *     &lt;/bc_node&gt;
     * &lt;/bc_node&gt;
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testShortestPath3() throws Exception {
        Map siteMap = TestHelper.getSiteMap("test_files/path/path3.xml");
        BreadCrumbNode node = new BreadCrumbNode("/Forums", "Forums");

        List pathList = Util.shortestPath(siteMap, node);

        assertNotNull("Expects not null path list", pathList);
        assertEquals("Expects only one shortest path found.", 1, pathList.size());

        List path = new LinkedList();
        path.add(new RegexNodeMatcher("MyMain", "/", "/"));
        path.add(new RegexNodeMatcher("Topics", "/topics", "/topics"));
        path.add(new RegexNodeMatcher("Forums", "/Forums", "/Forums"));

        assertTrue("Failed to get correct shortest path.", pathList.contains(path));

    }

    /**
     * <p>
     * Tests Util#shortestPath(Map,BreadCrumbNode) for accuracy.
     * </p>
     *
     * <p>
     * Expects one path be returned while there are two paths are available..
     * </p>
     *
     * <p>
     * The siteMap graph :
     * &lt;bc_node title=&quot;MyMain&quot; url=&quot;/&quot; pattern=&quot;/&quot;&gt;
     *     &lt;bc_node title=&quot;Develope&quot; url=&quot;/develope&quot; pattern=&quot;/develope&quot;&gt;
     *         &lt;bc_node title=&quot;Contest&quot; url=&quot;/contest&quot; pattern=&quot;/contest&quot;&gt;
     *             &lt;bc_node title=&quot;Forums&quot; url=&quot;/Forums&quot; pattern=&quot;/Forums&quot; /&gt;
     *         &lt;/bc_node&gt;
     *     &lt;/bc_node&gt;
     *     &lt;bc_node title=&quot;Topics&quot; url=&quot;/topics&quot; pattern=&quot;/topics&quot;&gt;
     *         &lt;bc_node title=&quot;Forums&quot; url=&quot;/Forums&quot; pattern=&quot;/Forums&quot; /&gt;
     *     &lt;/bc_node&gt;
     * &lt;/bc_node&gt;
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testShortestPath4() throws Exception {
        Map siteMap = TestHelper.getSiteMap("test_files/path/path4.xml");
        BreadCrumbNode node = new BreadCrumbNode("/Forums", "Forums");

        List pathList = Util.shortestPath(siteMap, node);

        assertNotNull("Expects not null path list", pathList);
        assertEquals("Expects only one shortest path found.", 1, pathList.size());

        List path = new LinkedList();
        path.add(new RegexNodeMatcher("MyMain", "/", "/"));
        path.add(new RegexNodeMatcher("Topics", "/topics", "/topics"));
        path.add(new RegexNodeMatcher("Forums", "/Forums", "/Forums"));

        assertTrue("Failed to get correct shortest path.", pathList.contains(path));
    }

    /**
     * <p>
     * Tests Util#shortestPath(Map,BreadCrumbNode) for accuracy.
     * </p>
     *
     * <p>
     * Expects two paths be returned while there are two paths are available..
     * </p>
     *
     * <p>
     * The siteMap graph :
     * &lt;bc_node title=&quot;MyMain&quot; url=&quot;/&quot; pattern=&quot;/&quot;&gt;
     *     &lt;bc_node title=&quot;Develop&quot; url=&quot;/develop&quot; pattern=&quot;/develop&quot;&gt;
     *         &lt;bc_node title=&quot;Forums&quot; url=&quot;/Forums&quot; pattern=&quot;/Forums&quot; /&gt;
     *     &lt;/bc_node&gt;
     *     &lt;bc_node title=&quot;Topics&quot; url=&quot;/topics&quot; pattern=&quot;/topics&quot;&gt;
     *        &lt;bc_node title=&quot;Forums&quot; url=&quot;/Forums&quot; pattern=&quot;/Forums&quot; /&gt;
     *     &lt;/bc_node&gt;
     * &lt;/bc_node&gt;
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testShortestPath5() throws Exception {
        Map siteMap = TestHelper.getSiteMap("test_files/path/path5.xml");
        BreadCrumbNode node = new BreadCrumbNode("/Forums", "Forums");

        List pathList = Util.shortestPath(siteMap, node);

        assertNotNull("Expects not null path list", pathList);
        assertEquals("Expects two shortest paths found.", 2, pathList.size());

        List path1 = new LinkedList();
        path1.add(new RegexNodeMatcher("MyMain", "/", "/"));
        path1.add(new RegexNodeMatcher("Topics", "/topics", "/topics"));
        path1.add(new RegexNodeMatcher("Forums", "/Forums", "/Forums"));

        List path2 = new LinkedList();
        path2.add(new RegexNodeMatcher("MyMain", "/", "/"));
        path2.add(new RegexNodeMatcher("Develop", "/develop", "/develop"));
        path2.add(new RegexNodeMatcher("Forums", "/Forums", "/Forums"));

        assertTrue("Failed to get correct shortest path.", pathList.contains(path1));
        assertTrue("Failed to get correct shortest path.", pathList.contains(path2));
    }

    /**
     * <p>
     * Tests Util#shortestPath(Map,BreadCrumbNode) for accuracy.
     * </p>
     *
     * <p>
     * Expects one path be returned while there are two paths are available.
     * Note, two terminal nodes.
     * </p>
     *
     * <p>
     * The siteMap graph :
     * &lt;bc_node title=&quot;MyMain&quot; url=&quot;/&quot; pattern=&quot;/&quot;&gt;
     *     &lt;bc_node title=&quot;Topics&quot; url=&quot;/topics&quot; pattern=&quot;/topics.*&quot;&gt;
     *         &lt;bc_node title=&quot;Forums&quot; url=&quot;/topics/forums&quot;
     *         pattern=&quot;/topics/forums&quot; /&gt;
     *     &lt;/bc_node&gt;
     * &lt;/bc_node&gt;
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testShortestPath6() throws Exception {
        Map siteMap = TestHelper.getSiteMap("test_files/path/path6.xml");
        BreadCrumbNode node = new BreadCrumbNode("/topics/forums", "Forums");

        List pathList = Util.shortestPath(siteMap, node);

        assertNotNull("Expects not null path list", pathList);
        assertEquals("Expects only one shortest path found.", 1, pathList.size());

        List path = new LinkedList();
        path.add(new RegexNodeMatcher("MyMain", "/", "/"));
        path.add(new RegexNodeMatcher("Topics", "/topics", "/topics.*"));

        assertTrue("Failed to get correct shortest path.", pathList.contains(path));
    }

    /**
     * <p>
     * Tests Util#shortestPath(Map,BreadCrumbNode) for accuracy.
     * </p>
     *
     * <p>
     * Expects two paths be returned while there are two paths are available.
     * Note, two terminal nodes.
     * </p>
     *
     * <p>
     * The siteMap graph :
     * &lt;bc_node title=&quot;MyMain&quot; url=&quot;/&quot; pattern=&quot;/&quot;&gt;
     *     &lt;bc_node title=&quot;Topics&quot; url=&quot;/topics&quot; pattern=&quot;/topics&quot;&gt;
     *          &lt;bc_node title=&quot;Forums&quot; url=&quot;/Forums&quot;
     *          pattern=&quot;/Forums[?]topics=.*&quot; /&gt;
     *     &lt;/bc_node&gt;
     * &lt;/bc_node&gt;
     * &lt;bc_node title=&quot;Main&quot; url=&quot;/main&quot; pattern=&quot;/main&quot;&gt;
     *     &lt;bc_node title=&quot;Discuss&quot; url=&quot;/discuss&quot; pattern=&quot;/discuss&quot;&gt;
     *         &lt;bc_node title=&quot;Questions&quot; url=&quot;/Forums&quot;
     *         pattern=&quot;/Forums[?]topics=.*&quot; /&gt;
     *     &lt;/bc_node&gt;
     * &lt;/bc_node&gt;
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testShortestPath7() throws Exception {
        Map siteMap = TestHelper.getSiteMap("test_files/path/path7.xml");
        BreadCrumbNode node = new BreadCrumbNode("/Forums?topics=dev", "Forums");

        List pathList = Util.shortestPath(siteMap, node);

        assertNotNull("Expects not null path list", pathList);
        assertEquals("Expects two shortest paths found.", 2, pathList.size());

        List path1 = new LinkedList();
        path1.add(new RegexNodeMatcher("MyMain", "/", "/"));
        path1.add(new RegexNodeMatcher("Topics", "/topics", "/topics"));
        path1.add(new RegexNodeMatcher("Forums", "/Forums", "/Forums[?]topics=.*"));

        List path2 = new LinkedList();
        path2.add(new RegexNodeMatcher("Main", "/main", "/main"));
        path2.add(new RegexNodeMatcher("Discuss", "/discuss", "/discuss"));
        path2.add(new RegexNodeMatcher("Questions", "/Forums", "/Forums[?]topics=.*"));

        assertTrue("Failed to get correct shortest path.", pathList.contains(path1));
        assertTrue("Failed to get correct shortest path.", pathList.contains(path2));
    }

    /**
     * <p>
     * Tests Util#shortestPath(Map,BreadCrumbNode) for accuracy.
     * </p>
     *
     * <p>
     * Expects one path be returned while there are two paths are available.
     * Note, loop contained in site map.
     * </p>
     *
     * <p>
     * The siteMap graph :
     * &lt;bc_node title=&quot;MyMain&quot; url=&quot;/&quot; pattern=&quot;/&quot;&gt;
     *     &lt;bc_node title=&quot;Topics&quot; url=&quot;/topics&quot;
     *     pattern=&quot;/topics&quot;&gt;
     *         &lt;bc_node title=&quot;Forums&quot; url=&quot;/Forums&quot;
     *         pattern=&quot;/Forums[?]topics=.*&quot;&gt;
     *             &lt;bc_node title=&quot;Discuss&quot; url=&quot;/discuss&quot;
     *             pattern=&quot;/discuss&quot; /&gt;
     *         &lt;/bc_node&gt;
     *     &lt;/bc_node&gt;
     *  &lt;/bc_node&gt;
     *  &lt;bc_node title=&quot;Main&quot; url=&quot;/main&quot; pattern=&quot;/main&quot;&gt;
     *     &lt;bc_node title=&quot;Discuss&quot; url=&quot;/discuss&quot; pattern=&quot;/discuss&quot;&gt;
     *         &lt;bc_node title=&quot;Questions&quot; url=&quot;/Forums&quot;
     *         pattern=&quot;/Forums[?]questions=.*&quot; /&gt;
     *         &lt;bc_node title=&quot;Topics&quot; url=&quot;/topics&quot; pattern=&quot;/topics&quot; /&gt;
     *     &lt;/bc_node&gt;
     * &lt;/bc_node&gt;
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testShortestPath8() throws Exception {
        Map siteMap = TestHelper.getSiteMap("test_files/path/path8.xml");
        BreadCrumbNode node = new BreadCrumbNode("/Forums?topics=444", "Forums");

        List pathList = Util.shortestPath(siteMap, node);

        assertNotNull("Expects not null path list", pathList);
        assertEquals("Expects only one shortest path found.", 1, pathList.size());

        List path = new LinkedList();
        path.add(new RegexNodeMatcher("MyMain", "/", "/"));
        path.add(new RegexNodeMatcher("Topics", "/topics", "/topics"));
        path.add(new RegexNodeMatcher("Forums", "/Forums", "/Forums[?]topics=.*"));

        assertTrue("Failed to get correct shortest path.", pathList.contains(path));
    }

    /**
     * <p>
     * Tests Util#shortestPath(Map,BreadCrumbNode) for accuracy.
     * </p>
     *
     * <p>
     * Expects one path be returned while there is only one path is available.
     * Note, the current node is a root node.
     * </p>
     *
     * <p>
     * The siteMap graph :
     * &lt;bc_node title=&quot;MyMain&quot; url=&quot;/&quot; pattern=&quot;/&quot;&gt;
     *     &lt;bc_node title=&quot;Topics&quot; url=&quot;/topics&quot; pattern=&quot;/topics&quot;&gt;
     *         &lt;bc_node title=&quot;Forums&quot; url=&quot;/Forums&quot; pattern=&quot;/Forums&quot; /&gt;
     *     &lt;/bc_node&gt;
     * &lt;/bc_node&gt;
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testShortestPath9() throws Exception {
        Map siteMap = TestHelper.getSiteMap("test_files/path/path3.xml");
        BreadCrumbNode node = new BreadCrumbNode("/", "MyMain");

        List pathList = Util.shortestPath(siteMap, node);

        assertNotNull("Expects not null path list", pathList);
        assertEquals("Expects only one shortest path found.", 1, pathList.size());

        List path = new LinkedList();
        path.add(new RegexNodeMatcher("MyMain", "/", "/"));

        assertTrue("Failed to get correct shortest path.", pathList.contains(path));
    }

    /**
     * <p>
     * Tests Util#shortestPath(Map,BreadCrumbNode) for failure.
     * It tests the case that when siteMap is null and expects IllegalArgumentException.
     * </p>
     */
    public void testShortestPath_NullSiteMap() {
        try {
            Util.shortestPath(null, new BreadCrumbNode("/main", "Main"));
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests Util#shortestPath(Map,BreadCrumbNode) for failure.
     * It tests the case that when node is null and expects IllegalArgumentException.
     * </p>
     */
    public void testShortestPath_NullNode() {
        try {
            Util.shortestPath(new HashMap(), null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests Util#getPageContextAttribute(PageContext,String,int,Class,boolean) for accuracy.
     * It tests the case that when the given attribute doesn't exist in pageContext and null is allowed.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testGetPageContextAttribute1() throws Exception {
        String name = "NAME";
        PageContext pageContext = new MockPageContext();

        Object obj = Util.getPageContextAttribute(pageContext, name, PageContext.PAGE_SCOPE, String.class, true);

        assertNull("Expects the attribute has null value.", obj);
    }

    /**
     * <p>
     * Tests Util#getPageContextAttribute(PageContext,String,int,Class,boolean) for failure.
     * It tests the case that when the given attribute doesn't exist in pageContext and null is not allowed.
     * Expects BreadCrumbException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testGetPageContextAttribute2() throws Exception {
        String name = "NAME";
        PageContext pageContext = new MockPageContext();

        try {
            Util.getPageContextAttribute(pageContext, name, PageContext.PAGE_SCOPE, String.class, false);
            fail("BreadCrumbException expected.");
        } catch (BreadCrumbException e) {
            //good
        }
    }

    /**
     * <p>
     * Tests Util#getPageContextAttribute(PageContext,String,int,Class,boolean) for failure.
     * It tests the case that when the given attribute has a non-string value and expects BreadCrumbException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testGetPageContextAttribute3() throws Exception {
        String name = "NAME";
        PageContext pageContext = new MockPageContext();
        pageContext.setAttribute(name, new StringBuffer("value"));

        try {
            Util.getPageContextAttribute(pageContext, name, PageContext.PAGE_SCOPE, String.class, false);
            fail("BreadCrumbException expected.");
        } catch (BreadCrumbException e) {
            //good
        }
    }

}