/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.nodeformatter;

import javax.servlet.jsp.PageContext;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbTrailTag;
import com.topcoder.web.ui.tag.breadcrumbtrail.MockPageContext;

import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.framework.Test;

/**
 * <p>
 * Unit test cases for TemplateNodeFormatterTag.
 * </p>
 *
 * <p>
 * For in-container tests, please see NodeFormatterTagInContainerTests.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0
 */
public class TemplateNodeFormatterTagTests extends TestCase {

    /**
     * <p>
     * Template file for testing.
     * </p>
     */
    private static final String TEMPLATE = "weblogic.jsp";

    /**
     * <p>
     * TemplateNodeFormatterTag instance for testing.
     * </p>
     */
    private TemplateNodeFormatterTag tag;

    /**
     * <p>
     * PageContext instance for testing.
     * </p>
     */
    private PageContext pageContext;

    /**
     * <p>
     * Setup test environment.
     * </p>
     *
     */
    protected void setUp() {
        pageContext = new MockPageContext();
        tag = new TemplateNodeFormatterTag(TEMPLATE);

        tag.setPageContext(pageContext);
    }

    /**
     * <p>
     * Tears down test environment.
     * </p>
     *
     */
    protected void tearDown() {
        tag = null;
        pageContext = null;
    }

    /**
     * <p>
     * Return all tests.
     * </p>
     *
     * @return all tests
     */
    public static Test suite() {
        return new TestSuite(TemplateNodeFormatterTagTests.class);
    }

    /**
     * <p>
     * Tests ctor TemplateNodeFormatterTag#TemplateNodeFormatterTag(String) for accuracy.
     * </p>
     */
    public void testCtor1() {
        assertNotNull("Failed to create new TemplateNodeFormatterTag instance.", tag);
    }

    /**
     * <p>
     * Tests ctor TemplateNodeFormatterTag#TemplateNodeFormatterTag(String) for accuracy.
     * It tests the case that when template is null and expects success.
     * </p>
     */
    public void testCtor1_NullTemplate() {
        tag = new TemplateNodeFormatterTag(null);
        assertNotNull("Failed to create new TemplateNodeFormatterTag instance.", tag);
        assertNull("Expects template is null.", tag.getTemplate());
    }

    /**
     * <p>
     * Tests ctor TemplateNodeFormatterTag#TemplateNodeFormatterTag(String) for accuracy.
     * It tests the case that when template is empty and expects success.
     * </p>
     */
    public void testCtor1_EmptyTemplate() {
        tag = new TemplateNodeFormatterTag("  ");
        assertNotNull("Failed to create new TemplateNodeFormatterTag instance.", tag);
        assertEquals("Failed to get correct template.", "  ", tag.getTemplate());
    }

    /**
     * <p>
     * Tests ctor TemplateNodeFormatterTag#TemplateNodeFormatterTag() for accuracy.
     * </p>
     */
    public void testCtor2() {
        assertNotNull("Failed to create new TemplateNodeFormatterTag instance.", new TemplateNodeFormatterTag());
    }

    /**
     * <p>
     * Tests TemplateNodeFormatterTag#getTemplate() for accuracy.
     * </p>
     */
    public void testGetTemplate() {
        assertEquals("Failed to get correct template.", TEMPLATE, tag.getTemplate());
    }

    /**
     * <p>
     * Tests TemplateNodeFormatterTag#setTemplate(String) for accuracy.
     * </p>
     */
    public void testSetTemplate() {
        tag.setTemplate("jboss.jsp");

        assertEquals("Failed to get correct template.", "jboss.jsp", tag.getTemplate());
    }

    /**
     * <p>
     * Tests TemplateNodeFormatterTag#setTemplate(String) for accuracy.
     * It tests the case that when template is null and expects success.
     * </p>
     */
    public void testSetTemplate_NullTemplate() {
        tag.setTemplate(null);

        assertNull("Expects template is null.", tag.getTemplate());
    }

    /**
     * <p>
     * Tests TemplateNodeFormatterTag#setTemplate(String) for accuracy.
     * It tests the case that when template is empty and expects success.
     * </p>
     */
    public void testSetTemplate_EmptyTemplate() {
        tag.setTemplate(" ");

        assertEquals("Failed to get correct template.", " ", tag.getTemplate());
    }

    /**
     * <p>
     * Tests HtmlAnchorNodeFormatterTag#doStartTag() for failure.
     * </p>
     *
     * <p>
     * It tests the case when BREADCRUMB_FORMAT_NODE attribute is not specified in pageContext and
     * expects BreadCrumbException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testDoStartTag_BreadCrumbException() throws Exception {
        try {
            tag.doStartTag();
            fail("BreadCrumbException expected.");
        } catch (BreadCrumbException e) {
            //good
        }
    }

    /**
     * <p>
     * Tests HtmlAnchorNodeFormatterTag#doStartTag() for failure.
     * Expects BreadCrumbException.
     * </p>
     *
     * <p>
     * It tests the case when BREADCRUMB_FORMAT_NODE attribute is not of BreadCrumbNode type in pageContext and
     * expects BreadCrumbException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testDoStartTag_BreadCrumbException2() throws Exception {
        try {
            pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, "node");
            tag.doStartTag();
            fail("BreadCrumbException expected.");
        } catch (BreadCrumbException e) {
            //good
        }
    }

    /**
     * <p>
     * Tests HtmlAnchorNodeFormatterTag#doStartTag() for failure.
     * Expects BreadCrumbException.
     * </p>
     *
     * <p>
     * It tests the case when BREADCRUMB_FORMAT_STYLE_CLASS attribute is not of String type in pageContext and
     * expects BreadCrumbException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testDoStartTag_BreadCrumbException3() throws Exception {
        try {
            pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, new StringBuffer("node"));
            tag.doStartTag();
            fail("BreadCrumbException expected.");
        } catch (BreadCrumbException e) {
            //good
        }
    }

}